local addonName, AT = ...
local DEBUG = false
local function _assert(aCnd, aMsg) if DEBUG then assert(aCnd, aMsg) end end

local Constants = AT.Constants
local Settings = AT.Settings

local BUFF_INFO = AT.Enums.BUFF_INFO
local NP_ORIENTATION = AT.Enums.NP_ORIENTATION
local PROFILE_TYPE = AT.Enums.PROFILE_TYPE

AT.nameplateMap = {}

local NameplateFrames = {}
NameplateFrames.createdRootFrames = {}
NameplateFrames.iconWidth = nil
NameplateFrames.iconOverallWidth = nil
NameplateFrames.iconHeight = nil
NameplateFrames.iconOverallHeight = nil
NameplateFrames.iconBorderSize = nil
NameplateFrames.iconGap = nil
NameplateFrames.texOrientation = nil

AT.NameplateFrames = NameplateFrames

function NameplateFrames:Init()
  Settings.OnProfileChanged:Add(self, self.UpdateSettings, PROFILE_TYPE.GENERAL)

  self:UpdateSettings()
end

function NameplateFrames:UpdateSettings()
  self.offsetX = Settings.npOffsetX
  self.offsetY = Settings.npOffsetY
  self.iconWidth = Settings.npIconWidth
  self.iconHeight= Settings.npIconHeight
  self.iconBorderSize = Settings.npIconBorderSize
  self.iconGap = Settings.npIconSpacing
  self.texOrientation = Settings.npIconOrientation
  self.iconOverallWidth = self.iconWidth + self.iconBorderSize * 2
  self.iconOverallHeight = self.iconHeight + self.iconBorderSize * 2

  for _, f in ipairs(self.createdRootFrames) do
    f.isUpToDate = false
    for _, c in ipairs(f.children) do
      c.isUpToDate = false
    end
  end
end

function NameplateFrames:ClearAll()
  for _, np in pairs(AT.nameplateMap) do
    self:Clear(np)
  end
end

function NameplateFrames:Clear(aNp)
  local frame = aNp[addonName]
    if frame then
      frame:Hide()
      for _, v in ipairs(frame.children) do
        v:Hide()
      end
    end
end

function NameplateFrames:Add(aNp, aUnit, aStealableBuffsInfo)
  local unitFrame = aNp.UnitFrame
  if not unitFrame or unitFrame:IsForbidden() then
    return
  end

  local frame = aNp[addonName]
  if not frame then
    frame = CreateFrame('Frame', nil, unitFrame)
    frame:EnableMouse(true)
    frame.children = {}
    aNp[addonName] = frame
    tinsert(self.createdRootFrames, frame)
  end

  local maxWidth = unitFrame.healthBar:GetWidth() * unitFrame.healthBar:GetScale()

  if not frame.isUpToDate then
    frame.isUpToDate = true
    frame:SetSize(maxWidth, self.iconHeight)
    frame:SetParent(AT.Utils:Ternary(Settings.npIconKeepOpaque, nil, aNp))
    frame:SetFrameStrata(Settings.npStrata)
  end

  local texX = 0
  for i, buffInfo in ipairs(aStealableBuffsInfo) do
    if buffInfo[BUFF_INFO.SHOW_IN_NAMEPLATE] then
      local texFrame = frame.children[i]
      if not texFrame then
        texFrame = CreateFrame('Frame', nil, frame)
        texFrame.tex = texFrame:CreateTexture(nil, 'OVERLAY')
        frame.children[i] = texFrame
      end

      if not texFrame.isUpToDate then
        texFrame.isUpToDate = true
        texFrame:ClearAllPoints()
        texFrame:SetSize(self.iconOverallWidth, self.iconOverallHeight)
        if self.iconBorderSize > 0 then
          local borderColor = Settings.npIconBorderColor
          texFrame:SetBackdrop({
            edgeFile = Constants.TEXTURE_BG,
            edgeSize = self.iconBorderSize,
          })
          texFrame:SetBackdropBorderColor(borderColor.r, borderColor.g, borderColor.b, borderColor.a)
        else
          texFrame:SetBackdropBorderColor(0, 0, 0, 0)
        end
        texFrame.tex:SetSize(self.iconWidth, self.iconHeight)
        texFrame.tex:ClearAllPoints()
        texFrame.tex:SetPoint('CENTER', texFrame, 0 , 0)
      end

      texFrame.tex:SetTexture(buffInfo[BUFF_INFO.ICON])
      texFrame.tex:SetAlpha(1)
      texFrame:Show()

      if self.texOrientation == NP_ORIENTATION.LEFT or self.texOrientation == NP_ORIENTATION.CENTER then
        texFrame:SetPoint('LEFT', frame, texX , 0)
      else
        texFrame:SetPoint('RIGHT', frame, -texX, 0)
      end

      texX = texX + self.iconOverallWidth + self.iconGap

      if texX + self.iconOverallWidth + self.iconGap >= maxWidth then
        break
      end
    end
  end

  _assert(texX ~= 0)
  texX = texX - self.iconOverallWidth - self.iconGap
  frame:ClearAllPoints()
  if self.texOrientation == NP_ORIENTATION.CENTER then
    frame:SetPoint('CENTER', unitFrame.healthBar, maxWidth / 2 - (texX + self.iconOverallWidth) / 2 + self.offsetX, self.offsetY - self.iconBorderSize)
  elseif self.texOrientation == NP_ORIENTATION.LEFT then
    frame:SetPoint('LEFT', unitFrame.healthBar, self.offsetX , self.offsetY - self.iconBorderSize)
  elseif self.texOrientation == NP_ORIENTATION.RIGHT then
    frame:SetPoint('RIGHT', unitFrame.healthBar, self.offsetX , self.offsetY - self.iconBorderSize)
  end

  frame:Show()
end
